@@ -240,7 +240,7 @@ def qj(x='',  # pylint: disable=invalid-name
 
       # Now, either we have set the log message, or we are ready to build it directly from x.
       if not log:
-        log = str(x)
+        log = qj.STR_FN(x)
       log = '(multiline log follows)\n%s' % log if '\n' in log else log
 
       padding_string = ''
@@ -265,7 +265,7 @@ def qj(x='',  # pylint: disable=invalid-name
 
       # If there's a lambda, run it and log it.
       if l:
-        log = str(l(x))
+        log = qj.STR_FN(l(x))
         log = '(multiline log follows)\n%s' % log if '\n' in log else log
         qj.LOG_FN('%s%s %s%s' % (qj.PREFIX, ' ' * len(prefix), qj._COLOR_LOG(),
                                  log))
@@ -294,15 +294,13 @@ def qj(x='',  # pylint: disable=invalid-name
       # If we requested tensorflow printing, wrap x in a tf.Print.
       if t:
         if (hasattr(x, '__module__') and
-            'tensorflow' in x.__module__ and
-            'tensorflow' in sys.modules):
-          tf = sys.modules['tensorflow']
+            'tensorflow' in x.__module__):
           prefix_spaces = ' ' * len(prefix)
           if 'session' in x.__module__:
             try:
               # pylint: disable=g-import-not-at-top
               try:
-                from tensorflow.python import debug as tf_debug
+                from tensorflow.compat.v1.python import debug as tf_debug
               except ImportError:
                 from google3.third_party.tensorflow.python import debug as tf_debug
               # pylint: enable=g-import-not-at-top
@@ -319,27 +317,54 @@ def qj(x='',  # pylint: disable=invalid-name
           else:
             qj.LOG_FN('%s%s %sWrapping return value in tf.Print operation.' %
                       (qj.PREFIX, qj._COLOR_LOG(), prefix_spaces))
-            x = tf.Print(
-                x, [tf.shape(x), x],
+            try:
+              tfprint = sys.modules['tensorflow'].Print
+              tfshape = sys.modules['tensorflow'].shape
+            except:  # pylint: disable=bare-except
+              try:
+                tf = sys.modules.get(
+                    'tensorflow.python',
+                    sys.modules.get('google3.third_party.tensorflow.python'))
+                tfprint = tf.ops.logging_ops.Print
+                tfshape = tf.ops.array_ops.shape
+              except:  # pylint: disable=bare-except
+                tfprint = lambda x, *_, **__: x
+                tfshape = lambda x: []
+
+            x = tfprint(
+                x, [tfshape(x), x],
                 message='%s%s%s%s' % (qj._COLOR_PREFIX(), qj.PREFIX, prefix[:-1],
                                       qj._COLOR_END()),
                 first_n=qj.MAX_FRAME_LOGS if t is True else int(t),
                 summarize=qj.MAX_FRAME_LOGS,
-                name='qj_print_%s_%d' % (func_name.split(' ')[-1], f.f_lineno))
+                name='qj_print_%s_%d_%s' % (
+                    func_name.split(' ')[-1], f.f_lineno,
+                    x.name.replace('/', '_').replace(':', '_') if hasattr(x, 'name') else ''))
 
       if tfc:
         if (hasattr(x, '__module__') and
-            'tensorflow' in x.__module__ and
-            'tensorflow' in sys.modules):
-          tf = sys.modules['tensorflow']
+            'tensorflow' in x.__module__):
+          try:
+            tfcheck = sys.modules['tensorflow'].check_numerics
+          except:  # pylint: disable=bare-except
+            try:
+              tf = sys.modules.get(
+                  'tensorflow.python',
+                  sys.modules.get('google3.third_party.tensorflow.python'))
+              tfcheck = tf.ops.gen_array_ops.check_numerics
+            except:  # pylint: disable=bare-except
+              tfcheck = lambda x, *_, **__: x
+
           prefix_spaces = ' ' * len(prefix)
           qj.LOG_FN('%s%s %sWrapping return value in tf.check_numerics.' %
                     (qj.PREFIX, qj._COLOR_LOG(), prefix_spaces))
-          x = tf.check_numerics(
+          x = tfcheck(
               x,
               message='%s%s%s%s' % (
                   qj._COLOR_PREFIX(), qj.PREFIX, prefix[:-1], qj._COLOR_END()),
-              name='qj_check_numerics_%s_%d' % (func_name.split(' ')[-1], f.f_lineno))
+              name='qj_check_numerics_%s_%d_%s' % (
+                  func_name.split(' ')[-1], f.f_lineno,
+                  x.name.replace('/', '_').replace(':', '_') if hasattr(x, 'name') else ''))
 
       # toc needs to be processed before tic, so that single call tic/toc works in loops.
       if toc:
@@ -446,7 +471,7 @@ def qj(x='',  # pylint: disable=invalid-name
       if r != _QJ_R_MAGIC:
         prefix = '%s:%s%s <%d>:' % (func_name, spaces, s or type(r), f.f_lineno)
         prefix_spaces = ' ' * len(prefix)
-        log = str(r)
+        log = qj.STR_FN(r)
         log = '(multiline log follows)\n%s' % log if '\n' in log else log
         qj.LOG_FN('%s%s %sOverridden return value: %s' % (qj.PREFIX, prefix_spaces,
                                                           qj._COLOR_LOG(), log))
@@ -530,6 +555,8 @@ qj.LOG_FN = lambda *args: logging.info(qj._COLOR_FN(*args))
 qj.MAX_FRAME_LOGS = 200
 qj.PREFIX = 'qj: '
 
+qj.STR_FN = str
+
 qj._FN_MAPS = {}
 qj._DEBUG_QJ = False
 
@@ -554,11 +581,48 @@ qj.make_global = lambda sym=qj, name='qj', mod=sys.modules[_builtin_module_name]
 
 # When running qj interactively (e.g., from a colab), automatically call
 # qj.make_global(), and also add a general print function, pr, to the interactive
-# module and set it as qj.LOG_FN.
+# module and set it as qj.LOG_FN. Also make sure to capture logs and format the
+# output cell appropriately if running in colab specifically.
+# TODO(iansf):Annoyingly slow for high frequency logging in colab.
 if not hasattr(sys.modules['__main__'], '__file__'):
+  try:
+    from colabtools import googlelog  # pylint: disable=g-import-not-at-top
+    from colabtools import outputformat  # pylint: disable=g-import-not-at-top
+    import multiprocessing  # pylint: disable=g-import-not-at-top
+    qj._last_output_format = _time.time()
+    _capture = googlelog.Capture()
+
+    def _start_capture():
+      if multiprocessing.current_process().name != 'MainProcess':
+        return False
+      if _capture._global_mode:
+        return False
+      _capture.enter_global_mode()
+      return True
+
+    def _end_capture():
+      if multiprocessing.current_process().name != 'MainProcess':
+        return
+      _capture.exit_global_mode()
+      cur_time = _time.time()
+      if qj._last_output_format + 3 < cur_time:
+        qj._last_output_format = cur_time
+        outputformat.word_wrap('1')
+        outputformat.max_output_height('1400')
+
+  except ImportError:
+    _start_capture = lambda: None
+    _end_capture = lambda: None
   qj.make_global()
+
+  def _colab_log_fn(*args):
+    captured = _start_capture()
+    logging.info(qj._COLOR_FN(*args))
+    if captured:
+      _end_capture()
+
   qj.LOG_FN = qj.make_global(
-      lambda *args: logging.info(qj._COLOR_FN(*args)),
+      _colab_log_fn,
       'pr', sys.modules['__main__'])
   qj.PREFIX_COLOR = qj._PREFIX_COLOR_NOTEBOOK
   qj.LOG_COLOR = qj._LOG_COLOR_NOTEBOOK
@@ -773,12 +837,14 @@ def _build_instruction_stack3(co, lasti):
 
     qj._DEBUG_QJ and qj.LOG_FN('%s\noriginal oparg_repr: %r\n     new oparg_repr: %r' % (opname, instr.argval, oparg_repr))
 
+    instr_arg = 0 if instr.arg is None else instr.arg
+
     stack_entry = _StackEntry(
-        _stack_effect3(instr.opname, instr.arg),
+        _stack_effect3(instr.opname, instr_arg),
         curr_i,
         curr_l,
         instr.opname,
-        instr.arg,
+        instr_arg,
         oparg_repr if isinstance(oparg_repr, list) else [str(oparg_repr)],
         [],  # children
     )
@@ -824,14 +890,15 @@ _STACK_EFFECTS3 = {
     'BINARY_SUBSCR': -1,
     'BINARY_FLOOR_DIVIDE': -1,
     'BINARY_TRUE_DIVIDE': -1,
+
     'INPLACE_FLOOR_DIVIDE': -1,
     'INPLACE_TRUE_DIVIDE': -1,
-
     'INPLACE_ADD': -1,
     'INPLACE_SUBTRACT': -1,
     'INPLACE_MULTIPLY': -1,
     'INPLACE_MATRIX_MULTIPLY': -1,
     'INPLACE_MODULO': -1,
+
     'STORE_SUBSCR': -3,
     'DELETE_SUBSCR': -2,
 
@@ -898,7 +965,6 @@ _STACK_EFFECTS3 = {
     'DELETE_FAST': 0,
     'STORE_ANNOTATION': -1,
 
-
     'LOAD_CLOSURE': 1,
     'LOAD_DEREF': 1,
     'LOAD_CLASSDEREF': 1,
@@ -910,6 +976,8 @@ _STACK_EFFECTS3 = {
     'GET_AITER': 0,
     'GET_ANEXT': 1,
     'GET_YIELD_FROM_ITER': 0,
+
+    'LOAD_METHOD': 1,
 }
 
 
@@ -947,6 +1015,8 @@ def _stack_effect3(op_code, oparg):
     return -oparg
   if op_code == 'CALL_FUNCTION':
     return -oparg
+  if op_code == 'CALL_METHOD':
+    return -oparg - 1
   if op_code == 'CALL_FUNCTION_KW':
     return -oparg - 1
   if op_code == 'CALL_FUNCTION_EX':
@@ -957,14 +1027,13 @@ def _stack_effect3(op_code, oparg):
     return -2 if (oparg == 3) else -1
   if op_code == 'FORMAT_VALUE':
     # If there's a fmt_spec on the stack we go from 2->1 else 1->1.
-    return -1 if (oparg & FVS_MASK) == FVS_HAVE_SPEC else 0
+    return -1 if (oparg & 0x4) == 0x4 else 0
   if op_code == 'EXTENDED_ARG':
     return 0  # EXTENDED_ARG just builds up a longer argument value for the next instruction (there may be multiple in a row?)
 
   return _STACK_EFFECTS3[op_code]
 
 
-
 #------------------------------------------------------------------------------
 # Python 2.7 Helpers
 #------------------------------------------------------------------------------
